﻿using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Net.Http.Formatting;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Web.Mvc;
using Newtonsoft.Json;
using tfsapi.client.Models;

namespace tfsapi.client.Controllers
{
    public class TFSReponse<T> where T : class
    {
        public int Count { get; set; }
        public List<T> Value { get; set; }
    }

    public class HomeController : Controller
    {
        private static string TfsURL = "http://{your server name here}/DefaultCollection/";
        private static string ProjectsURL = "_apis/projects?api-version=2.0";
        private static string ProjectDefinitionsURL = "{0}/_apis/build/definitions?api-version=2.0&type=build";
        private static string PostBuildURL = "{0}/_apis/build/builds?api-version=2.0";
        private static string BuildStatusURL = "{0}/_apis/build/builds/{1}?api-version=2.0";

        private static string PAT = "{your PAT here}";

        private HttpClient client = new HttpClient();


        private HttpClient CreateClient()
        {
            var client = new HttpClient();
            client.BaseAddress = new Uri(TfsURL);
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic",
                Convert.ToBase64String(Encoding.ASCII.GetBytes(string.Format("{0}:{1}", "", PAT))));
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            return client;
        }

        public async Task<ActionResult> Index()
        {
            var client = CreateClient();

            TFSReponse<TeamProject> projects = null;
            HttpResponseMessage response = await client.GetAsync(ProjectsURL);
            if (response.IsSuccessStatusCode)
            {
                projects = await response.Content.ReadAsAsync<TFSReponse<TeamProject>>();
            }
            return View(projects.Value);
        }

        public async Task<PartialViewResult> GetBuilds(string teamProjectName)
        {
            var client = CreateClient();

            TFSReponse<BuildDefinition> definitions = null;
            HttpResponseMessage response = await client.GetAsync(string.Format(ProjectDefinitionsURL, teamProjectName));
            if (response.IsSuccessStatusCode)
            {
                definitions = await response.Content.ReadAsAsync<TFSReponse<BuildDefinition>>();
            }
            ViewBag.TeamProjectName = teamProjectName;
            return PartialView("_BuildDefinitions", definitions.Value);
        }


        public async Task<PartialViewResult> RunBuild(string teamProjectName, string definitionId)
        {
            var client = CreateClient();
            var buildContent = new { definition = new { id = definitionId } };
            var buildjson = JsonConvert.SerializeObject(buildContent);

            var httpContent = new StringContent(buildjson, Encoding.UTF8, "application/json");

            BuildStatus status = null;
            HttpResponseMessage response = await client.PostAsync(string.Format(PostBuildURL, teamProjectName), httpContent);
            if (response.IsSuccessStatusCode)
            {
                status = await response.Content.ReadAsAsync<BuildStatus>();
            }
            ViewBag.TeamProjectName = teamProjectName;
            return PartialView("_BuildStatus", status);
        }

        public async Task<PartialViewResult> GetBuildStatus(string teamProjectName, BuildStatus build)
        {
            var client = CreateClient();

            BuildStatus status = null;
            HttpResponseMessage response = await client.GetAsync(string.Format(BuildStatusURL, teamProjectName, build.Id));
            if (response.IsSuccessStatusCode)
            {
                status = await response.Content.ReadAsAsync<BuildStatus>();
            }
            ViewBag.TeamProjectName = teamProjectName;
            return PartialView("_BuildStatus", status);
        }


    }
}